"""
Replace Latex math mode/\\begin{}\\end{} blocks with html img tag
after rendering

$Id: ReplaceInlineLatex.py,v 1.11 2001/09/07 14:21:16 rsbowman Exp $

TODO: img width and height attributes (pass from mogrify instead of using file).
    test a table/array
    use \\ref{shit} to make reference to equations.  Convert \\ref{shit} to the
    appropriate (number): add negative match to latexPattern for \\ref, do a
        s/// after generating an equation with a \\label.
    Separate vertical alignment characters & into a new
        \\end{equation}\\begin{equation} and then render the separate images in
        a table to acheive vertical alignment.

"""

import re, os
from DocumentTemplate.html_quote import html_quote
from util import basePath, fileNameFor, imagesPath, workDir
from string import strip, join, replace
from latexWrapper import runCommand

latexPattern = r'\$.+?\$|\\\[.+?\\\]'
#modeChangePattern = r'(\n?[ \t]*\\begin{ *(.+?) *}.*?\\end{ *\2 *}[ \t]*\n?)'
modeChangePattern = r'([ \t]*\\begin{ *(.+?) *}.*?\\end{ *\2 *}[ \t]*)'

errorMessage = """\n<hr><font size="-1" color="red">
Some or all expressions may not have rendered properly,
because Latex returned the following error:<br><pre>%s</pre></font>"""

reConsts = re.MULTILINE+re.DOTALL
def findLatexCode(text,
                  findInline=re.compile(latexPattern,
                                        reConsts).findall,
                  findModeChange=re.compile(modeChangePattern, 
                                            reConsts).findall):
    codeList = map(lambda x: x[0], findModeChange(text))
    codeList.extend(findInline(text))
    return codeList

def replaceInlineLatex(body, latexTemplate=None):
    from latexWrapper import renderNonexistingImages

    renumbered   = re.compile('\n?[ \t]*\\\\begin{ *(math|equation|eqnarray) *}')
    reunnumbered = re.compile('\n?[ \t]*\\\\begin{ *((math|equation|eqnarray)\\*|table|tabular|displaymath|array) *}')
    body = strip(body)
    latexCodeList = findLatexCode(body)
    eqnum = 0
    for code in latexCodeList:
        oldcode = code
        renummatch = renumbered.match(code)
        if renummatch:
            eqnum = eqnum + 1
            code = renumbered.sub('\\\\begin{\\1*}\n\\label{eq%d}' %(eqnum), code, 1)
            kind = renummatch.group(1)
            code = re.compile('\\\\end{ *%s *}' %(kind)).sub('\\\\end{%s*}' %(kind), code, 1)
            code = renumbered.sub('\\\\begin{\\1*}\n\\label{eq%d}' %(eqnum), code, 1)
            body = replace(body, oldcode, code, 1)
            latexCodeList[latexCodeList.index(oldcode)] = code
    errors = renderNonexistingImages(latexCodeList,
                                     latexTemplate=latexTemplate)
    
    if not errors:
        for code in latexCodeList:
            labelmatch = re.compile('\\\\label{eq(\\d+)}').search(code, 1)
            if labelmatch:
                eqnum = labelmatch.group(1)
                imageTag = '<a name="eq%s">' %(eqnum)                           \
                    + '<table width="100%"><tr><td align="center" width="95%">'                  \
                    + '\n<!--\n' + html_quote(code) + '\n-->\n'                 \
                    + getImageFor(code)                                         \
                    + '</td><td align="right">(%s)</td></tr></table></a>\n' %(eqnum)
            elif reunnumbered.match(code):
                imageTag = '<table width="100%"><tr><td align="center" width="95%">'             \
                    + '\n<!--\n' + html_quote(code) + '\n-->\n'                 \
                    + getImageFor(code)                                         \
                    + '</td><td width="5%" align="right">&nbsp;</td></tr></table>\n'
            else:
                imageTag = getImageFor(code)
            body = replace(body, code, imageTag, 1)

# We try to match the > and < from the preceeding and trailing tags, so as not
# to catch the alt="..." from a latex equation.  There should always be a
# preceeding and trailing tag.  (<html></html>)
# FIXME this is slow.  Use something like perl's m/\G/g to iterate over (n)
        for i in range(1,int(eqnum)+1):
            body = re.compile(r'(\( *(?:[Ee][Qq]\.)?(%d) *\))(?=[^>]*<)' %(i)).sub(r'<a href="#eq\2">(\2)</a>', body)

    else:
        body = "<pre>" + body + "</pre>" + errorMessage %(html_quote(join(errors,
                                                     '<br />')),)

    return body

def getImageFor(latexCode):    
    preamble, postamble = '', ''
    width, height = '', '' 
    imageFile = fileNameFor(latexCode, '.png')
    imageUrl = imagesPath + imageFile
    err, stdout, stderr = runCommand('file %s/%s' %(workDir, imageFile))
    if not err:
        sizeinfo = re.compile(r'PNG image data, (\d+) x (\d+)').search(stdout)
        if(sizeinfo != None): # This happens in case of a latex error.
            width = sizeinfo.group(1)
            height = sizeinfo.group(2)
    if latexCode[0] == '$':
        src = html_quote(re.compile(r'^\$(.*)\$$').sub('\\1', latexCode))
    else:
        src = "LatexWiki Image"

    return '%s<img alt="%s" align="middle" src="%s" width="%s" height="%s"/>%s' %(preamble,
                                                            src,
                                                            imageUrl,
                                                            width,
                                                            height,
                                                            postamble)
