###
###
###

import os, re, popen2, glob
from util import fileNameFor, workDir, imageExtension

class LatexSyntaxError(Exception): pass
class GhostscriptError(Exception): pass
class MogrifyError(Exception): pass

defaultLatexTemplate = r"""
\documentclass[10pt,notitlepage]{article}
\usepackage{amsmath}
\usepackage{amsfonts}
\usepackage[all]{xy}
\newenvironment{latex}{}{}
\begin{document}
\pagestyle{empty}
%s
\end{document}
"""

def log(msg, logFile='latexWikiLog'):
    open(logFile, 'a').write(msg)

def renderNonexistingImages(latexCodeList, **kw):
    """ take a list of strings of latex code, render the
    images that don't already exist.
    """
    from string import join

    errors = ()
    workingDir = kw.get('workDir', workDir)
    latexTemplate = (kw.get('latexTemplate',
                            defaultLatexTemplate) or
                     defaultLatexTemplate)
    
    def imageDoesNotExist(code):
        return not os.path.exists(fileNameFor(code, imageExtension))

    os.chdir(workingDir)
    codeToRender = filter(imageDoesNotExist, latexCodeList)
    
    if codeToRender:
        unifiedCode = join(codeToRender, '\n\\newpage\n')

        try:
            runLatex(unifiedCode, latexTemplate)
            runGhostscript(unifiedCode) 
        except LatexSyntaxError, data:
           errors = (str(data),)
           log('LatexSyntaxError: %s' %(errors))
           return errors

        imageFiles = glob.glob('%s*%s' %(fileNameFor(unifiedCode),
                                         imageExtension))
        cmdLine = 'mogrify -verbose -crop 0x0+2+2 '
        for image in imageFiles:
            cmdLine = cmdLine + ' ' + image
            log('mogrifying %s\n' %(image,))
        err, stdout, stderr = runCommand(cmdLine)
        if err:
            log('MogrifyError: %s\n%s' %(stderr, stdout))
            return stderr+'\n'+stdout

#        for code in codeToRender:
#            log("Attempting to run eqn2img -o %s\n"%(fileNameFor(code, imageExtension)))
#            mycode = re.compile(r'\$(.*)\$').search(code).group(1)
#            log(" For code: %s\n"%(mycode))
#            eqn2imgw, eqn2imgr = os.popen2("/usr/local/bin/eqn2img -v -o %s"%(fileNameFor(code, imageExtension)))
#            eqn2imgw.write(mycode)
#            eqn2imgw.close()
# Don't currently have a way to store or pass size data back to caller.
#        sizeinfo = re.compile(r'WIDTH=(\d+) HEIGHT=(\d)').search(eqn2imgr.read())
#        if(sizeinfo != None): # This happens in case of a latex error.
#            width = sizeinfo.group(1)
#            height = sizeinfo.group(2)

        for code, i in map(None, codeToRender, range(1, len(codeToRender)+1)):
            oldFileName = '%s-%03d.png' %(fileNameFor(unifiedCode), i)
            newFileName = fileNameFor(code, imageExtension)
            log('renaming %s to %s\n' %(oldFileName, newFileName))
            if os.path.exists(oldFileName):
                os.rename(oldFileName, newFileName)

        os.system('rm -f *.log *.aux *.tex *.pdf')
    return errors
    
def runCommand(cmdLine):
    program = popen2.Popen3(cmdLine, 1)
    program.tochild.close()
    status = program.wait()

    stderr = program.childerr.read()
    stdout = program.fromchild.read()
    rc = os.WEXITSTATUS(status)

    error = rc or not os.WIFEXITED(status)
    return error, stdout, stderr
   
def runMogrify(image):
    cmdLine = 'mogrify -verbose -crop 0x0+2+2 %s' %(image,)

    err, stdout, stderr = runCommand(cmdLine)
    if err:
        log('MogrifyError: %s\n%s' %(stderr, stdout))
        raise MogrifyError(stderr+'\n'+stdout)

def runLatex(code, latexTemplate):
    fileName = fileNameFor(code, '.tex')
    cmdLine = 'pdflatex --interaction errorstopmode %s' %(fileName,)

    file = open(fileName, 'w')
    file.write(latexTemplate %(code,))
    file.close()

    err, stdout, stderr = runCommand(cmdLine)
    
    if err:
        out = stderr + '\n' + stdout
        err = re.search('!.*\?', out, re.MULTILINE+re.DOTALL)
        if err:
            out = err.group(0)
        raise LatexSyntaxError(out)
    
def runGhostscript(unifiedCode):
    res, device = '130', 'pnggray'
    fName = fileNameFor(unifiedCode)
    input, output = fName+'.pdf', fName+'-%03d.png'
    
# BOB
#    outputeps = fName+'-%03d.eps'
#    cmdLine = 'gs -q -dSAFER -dDOINTERPOLATE ' + \
#              '-dNOPAUSE -sDEVICE=epswrite ' + \
#              '-sOutputFile=%s %s'
#    cmdLine = cmdLine %(outputeps, input)
#    err, stdout, stderr = runCommand(cmdLine)
#    cmdLine = 'gs -q -dSAFER -dDOINTERPOLATE ' + \
#              '-dNOPAUSE -sDEVICE=bbox %s'
#    cmdLine = cmdLine %(input)
#    err, stdout, stderr = runCommand(cmdLine)
#    log('Ghostscript command line: %s\n' %(cmdLine))
#    log('Ghostscript output: %s\n%s\n' %(stdout, stderr))
#    bboxes = re.split(r'\n(%%BoundingBox:(?: \d+)+\n%%HiResBoundingBox:(?: [0-9\.]+)+)\n', stderr)
#    num = 0
#    for bbox in bboxes:
#        num = num + 1
#        fname = outputeps %(num)
#        fname = workDir + '/' + fname
#        log('Working on file: %s\n' %(fname))
#        os.system('cp %s %s.orig' %(fname, fname))
#        myfile = open(fname, 'r+')
#        lines = myfile.readlines()
#        myfile.seek(0)
#        for line in lines:
#            if re.match('%%BoundingBox', line):
#                myfile.write(bbox + '\n')
#            elif re.match('%%HiResBoundingBox', line):
#                continue
#            else:
#                myfile.write(line)
#        myfile.truncate()
#        myfile.close()
#
#        log('Bbox for image %d:\n%s\n' %(num, bbox))
# /BOB

    cmdLine = 'gs -dSAFER -dDOINTERPOLATE -dTextAlphaBits=4 ' + \
              '-dGraphicsAlphaBits=4 -dNOPAUSE -r%s -sDEVICE=%s ' + \
              '-sOutputFile=%s %s'
    cmdLine = cmdLine %(res, device, output, input)
    err, stdout, stderr = runCommand(cmdLine)
    if err:
        raise GhostscriptError(stderr+'\n'+stdout)
